const express = require('express');
const mongoose = require('mongoose');
const cookieParser = require('cookie-parser');
const session = require('express-session');
const attachUser = require('./middleware/attachUser');
const expressLayouts = require('express-ejs-layouts');
const path = require('path');
const crypto = require('crypto');

// Routes
const authRoutes = require('./routes/auth');
const orderRoutes = require('./routes/order');
const paymentRoutes = require('./routes/payment');
const productRoutes = require('./routes/product');
const userRoutes = require('./routes/user');
const cartRoutes = require('./routes/cart');
const profileRoutes = require('./routes/profile');
const adminRoutes = require('./routes/admin')
const vendorRoutes = require('./routes/vendor');

const app = express();

require('dotenv').config();

// Session
app.use(session({
    secret: crypto.randomBytes(32).toString('hex'),
    resave: false,
    saveUninitialized: true,
}));

// Middleware
app.use(expressLayouts);
app.use(express.json());
app.use(express.urlencoded({ extended: true }));
app.use(cookieParser());
app.use(attachUser);

app.set('view engine', 'ejs');
app.set('views', path.join(__dirname, 'views'));

// MongoDB Connection
mongoose.connect(process.env.MONGODB_URI, { useNewUrlParser: true, useUnifiedTopology: true })
    .then(() => console.log('MongoDB connected'))
    .catch(err => console.error('MongoDB connection error:', err));

// Routes
app.use('/auth', authRoutes);
app.use('/orders', orderRoutes);
app.use('/payment', paymentRoutes);
app.use('/products', productRoutes);
app.use('/user', userRoutes);
app.use('/cart', cartRoutes);
app.use('/profile', profileRoutes);
app.use('/admin', adminRoutes);
app.use('/vendor', vendorRoutes);

app.get('/', (req, res) => {
    res.render('index', { title: 'Home Page'});
});

app.use(express.static(path.join('public')));

app.use((err, req, res, next) => {
    console.error(err.stack);
    res.status(500).send('Something went wrong!');
});


// Add users and products to the database
const bcrypt = require('bcrypt');
const User = require('./models/User');
const Product = require('./models/Product');


async function createUser(username, password, role) {
  const hashedPassword = await bcrypt.hash(password, 10);
  const user = new User({ username, password: hashedPassword, role });
  await user.save();
  console.log(`${username} created successfully`);
  return user;
}

async function createProduct(name, description, price, category, stock, image, vendorId, status) {
  const product = new Product({ name, description, price, category, stock, image, vendor: vendorId, status });
  await product.save();
  console.log(`${name} created successfully`);
  return product;
}

async function initializeData() {
  let vendor;

  try {
    try {
      vendor = await createUser("pwnsec_vendor", crypto.randomBytes(32).toString('hex'), "vendor");
      const admin = await createUser("pwnsec_admin", crypto.randomBytes(32).toString('hex'), "admin");
    } catch (error) {
      if (error.code === 11000) {
        console.log('Duplicate user detected');
        vendor = await User.findOne({ username: "pwnsec_vendor" });
      } else {
        console.error('Error creating user:', error);
      }
    }

    if (!vendor) {
      vendor = await User.findOne({ username: "pwnsec_vendor" });
      if (!vendor) {
        throw new Error('Vendor user not found');
      }
    }

    const products = [
      { name: 'PwnSec T-Shirt', description: 'T-Shirt', price: 10.99, category: 'Clothes', stock: 100, image: '/images/tshirt.webp', status: 'approved', vendor: vendor._id },
      { name: 'PwnSec Mug', description: 'Mug', price: 14.99, category: 'Accessories', stock: 100, image: '/images/mug.webp', status: 'approved', vendor: vendor._id },
      { name: 'PwnSec Hoodie', description: 'Hoodie', price: 30.99, category: 'Clothes', stock: 100, image: '/images/hoodie.jpg', status: 'approved', vendor: vendor._id },
      { name: 'Water Bottle', description: 'Water Bottle', price: 20.99, category: 'Accessories', stock: 200, image: '/images/water_bottle.jpg', status: 'approved', vendor: vendor._id },
      { name: 'Mouse Pad', description: 'Mouse Pad', price: 9.99, category: 'Accessories', stock: 300, image: '/images/mousepad.webp', status: 'approved', vendor: vendor._id },
      { name: 'Fake Flag', description: 'Just a fake flag', price: 10.99, category: 'Flag', stock: 100, image: '/images/fakeflag.webp', status: 'approved', vendor: vendor._id }
    ];

    for (const product of products) {
      try {
        await createProduct(product.name, product.description, product.price, product.category, product.stock, product.image, vendor._id, product.status);
      } catch (error) {
        if (error.code === 11000) {
          console.log(`Duplicate product detected: ${product.name}`);
        } else {
          console.error(`Error creating product ${product.name}:`, error);
        }
      }
    }

  } catch (error) {
    console.error('Error initializing data:', error);
  }
}

initializeData();

const PORT = process.env.PORT || 3000;
app.listen(PORT, () => console.log(`Server running on port ${PORT}`));
